#!/usr/bin/env python
# -*- coding: utf-8 -*-

from gimpfu import * 
import ctypes

def napln(image, drawable):

    # Zahájení sloučeného kroku
    pdb.gimp_image_undo_group_start(image)
    
    # Zjištění velikosti a vytvoření vrsty ze vstupního obrazu
    sirka = drawable.width
    vyska = drawable.height

    vrstva1 = pdb.gimp_layer_new_from_drawable(drawable, image)
       
    # Detekce hran Sobelovým filtrem       
    pdb.plug_in_edge(image, drawable, 1, 0, 0)
    
    # Výpočet hodnoty jasu
    monochrome = TRUE
    rr_gain = 0.299
    rg_gain = 0.299
    rb_gain = 0.299
    gr_gain = 0.587
    gg_gain = 0.587
    gb_gain = 0.587 
    br_gain = 0.114
    bg_gain = 0.114
    bb_gain = 0.114
    
    pdb.plug_in_colors_channel_mixer(image, drawable, monochrome, rr_gain, rg_gain, rb_gain, gr_gain, gg_gain, gb_gain, br_gain, bg_gain, bb_gain)

    # Převod na monochromatický obraz    
    pdb.gimp_image_convert_grayscale(image)

    #Zjištění bitové hloubky
    hloubka = drawable.bpp
    if pdb.gimp_drawable_has_alpha(drawable):
        hloubka = (256 ** (hloubka / 2)) - 1
    else:
        hloubka = (256 ** hloubka) - 1   
    
    # Zjištění průměrné hodnoty pixelu    
    histogram = pdb.gimp_drawable_histogram(drawable, 0, 0, 1)
    jdh = histogram[0] / hloubka
    jdh = jdh * 100

    # Vizualizace výsledku    
    sirka = int(sirka / 10)
    vyska = int(vyska / 10)   
    pdb.plug_in_pixelize2(image, drawable, sirka, vyska)
    
    pdb.gimp_image_convert_rgb(image)
    
    vrstva2 = pdb.gimp_layer_new_from_drawable(drawable, image)
    pdb.gimp_layer_set_opacity(vrstva2, 85)
    pdb.gimp_image_insert_layer(image, vrstva1, None, -1)
    pdb.gimp_image_insert_layer(image, vrstva2, None, -1)    
    
    # Oznámení hodnoty náplně
    hlaska = " Grafická náplň mapy: " + str(round(jdh, 1)) + " %"
    ctypes.windll.user32.MessageBoxW(0, unicode(hlaska, "utf-8"), u"VÝSLEDEK MĚŘENÍ", 0x40)
    
    # Ukončení sloučeného kroku
    pdb.gimp_image_undo_group_end(image)
    
register(
    "NAPLNMAPY_1-0",
    "Experimentální nástroj pro určování grafické náplně map založený na detekci hran",
    "Experimentální nástroj pro určování grafické náplně map založený na detekci hran pomocí Sobelova filtru, vyvinutý na Katedře geoinformatiky Univerzity Palackého v Olomouci v rámci výzkumu metrik pro výpočet grafické náplně map. Pro vzájemnou porovnatelnost se doporučuje využít rastry reprezentující mapu v originálních rozměrech a s rozlišením 100 DPI.",
    "Radek Barvíř", "CC BY-SA", "v 1.0, build 200925, 2020",
    "NÁPLŇ MAPY 1.0",
    "RGB*", 
    [
        (PF_IMAGE, "image", "takes current image", None),
        (PF_DRAWABLE, "drawable", "Input layer", None)
    ],
    [],
    napln, menu="<Image>/Filters/Edge-Detect")

main()